@@ -1,7 +1,7 @@
 /*
  *  ConfigDriver.c
  *
- *  $Id: ConfigDriver.c,v 1.7 2004/06/22 11:25:48 source Exp $
+ *  $Id: ConfigDriver.c,v 1.8 2005/05/12 12:27:14 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -69,7 +69,7 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 #include <iodbc_error.h>
 
 #include "gui.h"
