@@ -1,7 +1,7 @@
 /*
  *  gui.h
  *
- *  $Id: gui.h,v 1.6 2004/06/22 11:25:48 source Exp $
+ *  $Id: gui.h,v 1.7 2005/05/12 12:27:14 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -70,10 +70,10 @@
 
 #ifdef _MACX
 #  include <iODBC/iodbc.h>
-#  include <iODBCinst/iodbcinst.h>
+#  include <iODBCinst/odbcinst.h>
 #else
 #  include <iodbc.h>
-#  include <iodbcinst.h>
+#  include <odbcinst.h>
 #endif
 
 #if defined(__BEOS__)
