@@ -1,7 +1,7 @@
 /*
  *  sqlext.h
  *
- *  $Id: sqlext.h,v 1.8 2004/09/28 13:59:42 source Exp $
+ *  $Id: sqlext.h,v 1.9 2005/08/15 21:04:15 source Exp $
  *
  *  ODBC defines (ext)
  *
@@ -2562,6 +2562,6 @@ SQLRETURN SQL_API SQLAllocHandleStd (
 }
 #endif
 
-#include "sqlucode.h"
+#include <sqlucode.h>
 
 #endif	/* _SQLEXT_H */
