@@ -1,7 +1,7 @@
 /*
  *  sqltypes.h
  *
- *  $Id: sqltypes.h,v 1.17 2004/12/15 14:39:54 source Exp $
+ *  $Id: sqltypes.h,v 1.18 2005/10/08 11:39:53 source Exp $
  *
  *  ODBC typedefs
  *
@@ -389,7 +389,8 @@ typedef unsigned short SQLWCHAR;
 #else
 #  include <stdlib.h>
 
-#  if defined(_WCHAR_T)			|| \
+#  if defined(__cplusplus)		|| \
+      defined(_WCHAR_T)			|| \
       defined(_WCHAR_T_DEFINED)		|| \
       defined(_WCHAR_T_DECLARED)        || \
       defined(_BSD_WCHAR_T_DEFINED_)
