@@ -1,7 +1,7 @@
 /*
  *  connect.c
  *
- *  $Id: connect.c,v 1.32 2004/12/15 15:39:04 source Exp $
+ *  $Id: connect.c,v 1.40 2005/11/04 13:08:54 source Exp $
  *
  *  Connect (load) driver
  *
@@ -77,7 +77,7 @@
 #include <sqlext.h>
 #include <sqlucode.h>
 #include <iodbcext.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include <dlproc.h>
 
@@ -90,7 +90,7 @@
 
 #include <unicode.h>
 
-#if defined(__APPLE__) && defined(GUI)
+#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
 #include <Carbon/Carbon.h>
 #endif
 
@@ -115,6 +115,106 @@ extern wchar_t * _iodbcdm_getkeyvalinstrw (wchar_t *cnstr, int cnlen,
     wchar_t *keywd, wchar_t *value, int size);
 
 
+#define CHECK_DRVCONN_DIALBOXW(path) \
+  { \
+    if ((handle = DLL_OPEN(path)) != NULL) \
+      { \
+        if (DLL_PROC(handle, "_iodbcdm_drvconn_dialboxw") != NULL) \
+          { \
+            DLL_CLOSE(handle); \
+            retVal = TRUE; \
+            goto quit; \
+          } \
+        else \
+          { \
+            if (DLL_PROC(handle, "_iodbcdm_drvconn_dialbox") != NULL) \
+              { \
+                DLL_CLOSE(handle); \
+                retVal = TRUE; \
+                goto quit; \
+              } \
+          } \
+        DLL_CLOSE(handle); \
+      } \
+  }
+
+
+
+static BOOL
+_iodbcdm_CheckDriverLoginDlg (
+    SQLPOINTER  drv,
+    SQLPOINTER  dsn,
+    UCHAR       waMode
+)
+{
+  wchar_t *szDSN = NULL;
+  wchar_t *szDriver = NULL;
+  wchar_t tokenstr[4096];
+  char *_szdriver_u8 = NULL;
+  char drvbuf[4096] = { L'\0'};
+  HDLL handle;
+  BOOL retVal = FALSE;
+
+  if (waMode != 'W')
+   {
+     szDSN = dm_SQL_A2W((SQLCHAR *)dsn, SQL_NTS);
+     szDriver = dm_SQL_A2W((SQLCHAR *)drv, SQL_NTS);
+   }
+
+  /* Check if the driver is provided */
+  if (szDriver == NULL)
+    {
+      SQLSetConfigMode (ODBC_BOTH_DSN);
+      SQLGetPrivateProfileStringW (L"ODBC Data Sources",
+        szDSN && szDSN[0] != L'\0' ? szDSN : L"default",
+        L"", tokenstr, sizeof (tokenstr)/sizeof(wchar_t), NULL);
+      szDriver = tokenstr;
+    }
+
+  /* Call the iodbcdm_drvconn_dialbox */
+  _szdriver_u8 = dm_SQL_W2A (szDriver, SQL_NTS);
+
+  SQLSetConfigMode (ODBC_USER_DSN);
+  if (!access (_szdriver_u8, X_OK))
+    { CHECK_DRVCONN_DIALBOXW (_szdriver_u8); }
+  if (SQLGetPrivateProfileString (_szdriver_u8, "Driver", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString (_szdriver_u8, "Setup", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString ("Default", "Driver", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString ("Default", "Setup", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+
+
+  SQLSetConfigMode (ODBC_SYSTEM_DSN);
+  if (!access (_szdriver_u8, X_OK))
+    { CHECK_DRVCONN_DIALBOXW (_szdriver_u8); }
+  if (SQLGetPrivateProfileString (_szdriver_u8, "Driver", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString (_szdriver_u8, "Setup", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString ("Default", "Driver", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+  if (SQLGetPrivateProfileString ("Default", "Setup", "", drvbuf,
+    sizeof (drvbuf), "odbcinst.ini"))
+    { CHECK_DRVCONN_DIALBOXW (drvbuf); }
+
+quit:
+
+  MEM_FREE (_szdriver_u8);
+
+  return retVal;
+}
+
+
 static SQLRETURN
 _iodbcdm_SetConnectOption_init (
     SQLHDBC		  hdbc,
@@ -294,12 +394,13 @@ _iodbcdm_driverload (
   CONN (pdbc, hdbc);
   ENVR (penv, NULL);
   GENV (genv, NULL);
-  HDLL hdll;
+  HDLL hdll = SQL_NULL_HDLL;
   HPROC hproc;
   SQLRETURN retcode = SQL_SUCCESS;
   sqlstcode_t sqlstat = en_00000;
+  char *driverbuf[1024];
 
-  if (path == NULL || ((char*)path)[0] == '\0')
+  if (path == NULL || ((char *) path)[0] == '\0')
     {
       PUSHSQLERR (pdbc->herr, en_IM002);
       return SQL_ERROR;
@@ -310,11 +411,41 @@ _iodbcdm_driverload (
       return SQL_INVALID_HANDLE;
     }
 
+  /*
+   *  If path does not start with / or ., we may have a symbolic driver name
+   */
+  if (!(path[0] == '/' || path[0] == '.'))
+    {
+      char *tmp_path = NULL;
+
+      /*
+       *  Remove curly braces
+       */
+      if (path[0] == '{')
+	{
+	  tmp_path = strdup (path);
+	  if (tmp_path[strlen (path) - 1] == '}')
+	    tmp_path[strlen (path) - 1] = '\0';
+	  path = &tmp_path[1];
+	}
+
+      /*
+       *  Hopefully the driver was registered under that name in the 
+       *  odbcinst.ini file
+       */
+      if (SQLGetPrivateProfileString ((char *) path, "Driver", "",
+	      driverbuf, sizeof (driverbuf), "odbcinst.ini") && driverbuf[0])
+	path = driverbuf;
+
+      if (tmp_path)
+	free (tmp_path);
+    }
+
   genv = (GENV_t *) pdbc->genv;
 
   /* This will either load the driver dll or increase its reference count */
   hdll = _iodbcdm_dllopen ((char *) path);
-  
+
   /* Set flag if it is safe to unload the driver after use */
   if (unload_safe)
     _iodbcdm_safe_unload (hdll);
@@ -1167,6 +1298,13 @@ SQLDriverConnect_Internal (
   char *_dsn_u8 = NULL;
   char *_drv_u8 = NULL;
   UWORD config;
+  BOOL bCallDmDlg = FALSE;
+#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
+  CFStringRef libname = NULL;
+  CFBundleRef bundle = NULL;
+  CFURLRef liburl = NULL;
+  char name[1024] = { 0 };
+#endif
 
   HPROC dialproc = SQL_NULL_HPROC;
 
@@ -1194,16 +1332,16 @@ SQLDriverConnect_Internal (
 
   if (waMode != 'W')
     {
-      drv = _iodbcdm_getkeyvalinstr ((char *)szConnStrIn, cbConnStrIn,
+      drv = _iodbcdm_getkeyvalinstr ((char *) szConnStrIn, cbConnStrIn,
 	  "DRIVER", (char *) drvbuf, sizeof (drvbuf));
-      dsn = _iodbcdm_getkeyvalinstr ((char *)szConnStrIn, cbConnStrIn,
+      dsn = _iodbcdm_getkeyvalinstr ((char *) szConnStrIn, cbConnStrIn,
 	  "DSN", (char *) dsnbuf, sizeof (dsnbuf));
     }
   else
     {
-      drv = _iodbcdm_getkeyvalinstrw ((wchar_t *)szConnStrIn, cbConnStrIn,
+      drv = _iodbcdm_getkeyvalinstrw ((wchar_t *) szConnStrIn, cbConnStrIn,
 	  L"DRIVER", drvbuf, sizeof (drvbuf) / sizeof (SQLWCHAR));
-      dsn = _iodbcdm_getkeyvalinstrw ((wchar_t *)szConnStrIn, cbConnStrIn,
+      dsn = _iodbcdm_getkeyvalinstrw ((wchar_t *) szConnStrIn, cbConnStrIn,
 	  L"DSN", dsnbuf, sizeof (dsnbuf) / sizeof (SQLWCHAR));
     }
 
@@ -1229,53 +1367,101 @@ SQLDriverConnect_Internal (
       /* Get data source dialog box function from
        * current executable */
       /* Not really sure here, but should load that from the iodbcadm */
-      hdll = _iodbcdm_dllopen ("libiodbcadm.so");
+      if (waMode == 'A')
+	strncpy ((char *) prov, szConnStrIn, sizeof (prov));
+      else
+	wcsncpy (prov, szConnStrIn, sizeof (prov) / sizeof (wchar_t));
 
-      /* Check if there is a DSN provided */
-      if (dsn == NULL && drv == NULL)
+      if (!dsn && !drv)
+        bCallDmDlg = TRUE;
+      else if ( _iodbcdm_CheckDriverLoginDlg(drv, dsn, waMode) == FALSE)
+        bCallDmDlg = TRUE;
+
+      /* not call iODBC function "iodbcdm_drvconn_dialbox", if there is
+       * the function "_iodbcdm_drvconn_dialbox" in the odbc driver,
+       * odbc driver must call its function itself
+       */
+      if (!bCallDmDlg)
+        break;
+
+      ODBC_UNLOCK ();
+#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
+      bundle = CFBundleGetBundleWithIdentifier (CFSTR ("org.iodbc.core"));
+      if (bundle)
+	{
+	  /* Search for the drvproxy library */
+	  liburl =
+	      CFBundleCopyResourceURL (bundle, CFSTR ("iODBCadm.bundle"),
+	      NULL, NULL);
+	  if (liburl
+	      && (libname =
+		  CFURLCopyFileSystemPath (liburl, kCFURLPOSIXPathStyle)))
+	    {
+	      CFStringGetCString (libname, name, sizeof (name),
+		  kCFStringEncodingASCII);
+	      STRCAT (name, "/Contents/MacOS/iODBCadm");
+	      hdll = _iodbcdm_dllopen (name);
+	    }
+	  if (liburl)
+	    CFRelease (liburl);
+	  if (libname)
+	    CFRelease (libname);
+	}
+#else
+      hdll = _iodbcdm_dllopen ("libiodbcadm.so");
+#endif
+
+      if (!hdll)
+	break;
+
+      if (waMode != 'W')
+	dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialbox");
+      else
+	dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialboxw");
+
+      if (dialproc == SQL_NULL_HPROC)
+	{
+	  sqlstat = en_IM008;
+	  break;
+	}
+
+      retcode = dialproc (hwnd,	/* window or display handle */
+	  prov,			/* input/output dsn buf */
+	  sizeof (prov) / (waMode == 'A' ? 1 : sizeof (SQLWCHAR)),	/* buf size */
+	  &sqlstat,		/* error code */
+	  fDriverCompletion,	/* type of completion */
+	  &config);		/* config mode */
+
+
+      ODBC_LOCK ();
+
+      if (retcode != SQL_SUCCESS)
+	{
+	  if (retcode != SQL_NO_DATA_FOUND)
+	    PUSHSQLERR (pdbc->herr, sqlstat);
+	  goto end;
+	}
+
+      connStrIn = szConnStrIn = prov;
+
+      /*
+       * Recalculate length of szConnStrIn if needed, as it may have been
+       * changed by iodbcdm_drvconn_dialbox
+       */
+      if (cbConnStrIn != SQL_NTS)
 	{
 	  if (waMode != 'W')
-	    dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialbox");
+	    cbConnStrIn = STRLEN (szConnStrIn);
 	  else
-	    dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialboxw");
-
-	  if (dialproc == SQL_NULL_HPROC)
-	    {
-	      sqlstat = en_IM008;
-	      break;
-	    }
-
-	  retcode = dialproc (hwnd,	/* window or display handle */
-	      prov,		/* input/output dsn buf */
-	      sizeof (prov) / sizeof (SQLWCHAR),	/* buf size */
-	      &sqlstat,		/* error code */
-	      fDriverCompletion,	/* type of completion */
-	      &config);		/* config mode */
-
-	  if (retcode != SQL_SUCCESS)
-	    {
-	      if (retcode != SQL_NO_DATA_FOUND)
-		PUSHSQLERR (pdbc->herr, sqlstat);
-	      goto end;
-	    }
-
-	  connStrIn = szConnStrIn = prov;
-
-	  if (cbConnStrIn == SQL_NTS)
-	    {
-	      if (waMode != 'W')
-		cbConnStrIn = STRLEN (szConnStrIn);
-	      else
-		cbConnStrIn = WCSLEN (szConnStrIn);
-	    }
-
-	  if (waMode != 'W')
-	    dsn = _iodbcdm_getkeyvalinstr ((char *) prov, STRLEN (prov),
-		"DSN", (char *) dsnbuf, sizeof (dsnbuf));
-	  else
-	    dsn = _iodbcdm_getkeyvalinstrw (prov, WCSLEN (prov),
-		L"DSN", dsnbuf, sizeof (dsnbuf) / sizeof (SQLWCHAR));
+	    cbConnStrIn = WCSLEN (szConnStrIn);
 	}
+
+      if (waMode != 'W')
+	dsn = _iodbcdm_getkeyvalinstr ((char *) prov, STRLEN (prov),
+	    "DSN", (char *) dsnbuf, sizeof (dsnbuf));
+      else
+	dsn = _iodbcdm_getkeyvalinstrw (prov, WCSLEN (prov),
+	    L"DSN", dsnbuf, sizeof (dsnbuf) / sizeof (SQLWCHAR));
       break;
 
     default:
@@ -1293,7 +1479,7 @@ SQLDriverConnect_Internal (
     {
       if (dsn != NULL)
 	{
-          dsn = _dsn_u8 = (char *) dm_SQL_WtoU8((SQLWCHAR*)dsn, SQL_NTS);
+	  dsn = _dsn_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) dsn, SQL_NTS);
 	  if (dsn == NULL)
 	    {
 	      PUSHSQLERR (pdbc->herr, en_S1001);
@@ -1303,7 +1489,7 @@ SQLDriverConnect_Internal (
 
       if (drv != NULL)
 	{
-          drv = _drv_u8 = (char *) dm_SQL_WtoU8((SQLWCHAR*)drv, SQL_NTS);
+	  drv = _drv_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) drv, SQL_NTS);
 	  if (drv == NULL)
 	    {
 	      PUSHSQLERR (pdbc->herr, en_S1001);
@@ -1321,7 +1507,8 @@ SQLDriverConnect_Internal (
   else
     /* if you want tracing, you must use a DSN */
     {
-      setopterr |= _iodbcdm_con_settracing (pdbc, (SQLCHAR *) dsn, SQL_NTS, waMode);
+      setopterr |=
+	  _iodbcdm_con_settracing (pdbc, (SQLCHAR *) dsn, SQL_NTS, waMode);
     }
 
   /*
@@ -1330,8 +1517,8 @@ SQLDriverConnect_Internal (
   thread_safe = 1;		/* Assume driver is thread safe */
 
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if (SQLGetPrivateProfileString ((char *) dsn, "ThreadManager", "", 
-	buf, sizeof (buf), "odbc.ini")
+  if (SQLGetPrivateProfileString ((char *) dsn, "ThreadManager", "",
+	  buf, sizeof (buf), "odbc.ini")
       && (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
     {
       thread_safe = 0;		/* Driver needs a thread manager */
@@ -1343,8 +1530,8 @@ SQLDriverConnect_Internal (
   unload_safe = 0;		/* Assume driver is not unload safe */
 
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if (SQLGetPrivateProfileString ((char *) dsn, "UnloadSafe", "", 
-	buf, sizeof (buf), "odbc.ini")
+  if (SQLGetPrivateProfileString ((char *) dsn, "UnloadSafe", "",
+	  buf, sizeof (buf), "odbc.ini")
       && (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
     {
       unload_safe = 1;
@@ -1356,8 +1543,8 @@ SQLDriverConnect_Internal (
   if (drv == NULL || *(char *) drv == '\0')
     {
       SQLSetConfigMode (ODBC_BOTH_DSN);
-      if (SQLGetPrivateProfileString ((char *) dsn, "Driver", "", 
-		buf, sizeof (buf), "odbc.ini") != 0)
+      if (SQLGetPrivateProfileString ((char *) dsn, "Driver", "",
+	      buf, sizeof (buf), "odbc.ini") != 0)
 	{
 	  drv = buf;
 	}
@@ -1371,7 +1558,9 @@ SQLDriverConnect_Internal (
       return SQL_ERROR;
     }
 
-  retcode = _iodbcdm_driverload ((char *) drv, pdbc, thread_safe, unload_safe, waMode);
+  retcode =
+      _iodbcdm_driverload ((char *) drv, pdbc, thread_safe, unload_safe,
+      waMode);
 
   MEM_FREE (_dsn_u8);
   MEM_FREE (_drv_u8);
@@ -1454,12 +1643,14 @@ SQLDriverConnect_Internal (
       if (waMode != 'W')
 	{
 	  /* ansi<=unicode */
-          dm_StrCopyOut2_W2A ((SQLWCHAR *) connStrOut, (SQLCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
+	  dm_StrCopyOut2_W2A ((SQLWCHAR *) connStrOut,
+	      (SQLCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
 	}
       else
 	{
 	  /* unicode<=ansi */
-          dm_StrCopyOut2_A2W ((SQLCHAR *) connStrOut, (SQLWCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
+	  dm_StrCopyOut2_A2W ((SQLCHAR *) connStrOut,
+	      (SQLWCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
 	}
     }
 
