@@ -1,14 +1,32 @@
 /*
  *  dlf.c
  *
- *  $Id: dlf.c,v 1.12 2001/06/06 12:33:01 source Exp $
+ *  $Id: dlf.c,v 1.21 2005/07/29 12:15:11 source Exp $
  *
  *  Dynamic Library Loader (mapping to SVR4)
  *
  *  The iODBC driver manager.
  *  
  *  Copyright (C) 1995 by Ke Jin <kejin@empress.com> 
+ *  Copyright (C) 1996-2002 by OpenLink Software <iodbc@openlinksw.com>
+ *  All Rights Reserved.
  *
+ *  This software is released under the terms of either of the following
+ *  licenses:
+ *
+ *      - GNU Library General Public License (see LICENSE.LGPL) 
+ *      - The BSD License (see LICENSE.BSD).
+ *
+ *  While not mandated by the BSD license, any patches you make to the
+ *  iODBC source code may be contributed back into the iODBC project
+ *  at your discretion. Contributions will benefit the Open Source and
+ *  Data Access community as a whole. Submissions may be made at:
+ *
+ *      http://www.iodbc.org
+ *
+ *
+ *  GNU Library Generic Public License Version 2
+ *  ============================================
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
  *  License as published by the Free Software Foundation; either
@@ -22,6 +40,35 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *
+ *  The BSD License
+ *  ===============
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions
+ *  are met:
+ *
+ *  1. Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *  2. Redistributions in binary form must reproduce the above copyright
+ *     notice, this list of conditions and the following disclaimer in
+ *     the documentation and/or other materials provided with the
+ *     distribution.
+ *  3. Neither the name of OpenLink Software Inc. nor the names of its
+ *     contributors may be used to endorse or promote products derived
+ *     from this software without specific prior written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OPENLINK OR
+ *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <dlf.h>
@@ -183,7 +230,7 @@ static char *errmsg = 0;
 
 static void
 init_htab (hent_t * ht)
-/* initate a hashing table */
+/* initialize a hashing table */
 {
   int i;
 
@@ -288,7 +335,7 @@ slot_alloc (char *sym)
 
   ent = (slot_t *) malloc (sizeof (slot_t));
 
-  ent->sym = (char *) malloc (strlen (sym) + 1);
+  ent->sym = (char *) malloc (STRLEN (sym) + 1);
 
   if (!ent->sym)
     {
@@ -296,7 +343,7 @@ slot_alloc (char *sym)
       return 0;
     }
 
-  strcpy (ent->sym, sym);
+  STRCPY (ent->sym, sym);
 
   return ent;
 }
@@ -342,7 +389,7 @@ dlopen (char *file, int mode)
   if (!pobj)
     return 0;
 
-  pobj->path = (char *) malloc (strlen (file) + 1);
+  pobj->path = (char *) malloc (STRLEN (file) + 1);
 
   if (!pobj->path)
     {
@@ -350,7 +397,7 @@ dlopen (char *file, int mode)
       return 0;
     }
 
-  strcpy (pobj->path, file);
+  STRCPY (pobj->path, file);
 
   pobj->dev = st.st_dev;
   pobj->ino = st.st_ino;
@@ -482,7 +529,7 @@ dlsym (void *hdl, char *sym)
    * manager. 2. applications always pass driver's full 
    * path name. 3. if driver itself don't support 
    * SQLGetFunctions(), call it with SQL_ALL_FUNCTIONS
-   * as flag immidately after SQLConnect(), SQLDriverConnect()
+   * as flag immediately after SQLConnect(), SQLDriverConnect()
    * and SQLBrowseConnect() to force the driver manager
    * resolving all will be used symbols. 
    */
@@ -500,13 +547,13 @@ dlsym (void *hdl, char *sym)
    * is always 0. While on 4.x it will be 32. 
    * On AIX 4.x, if the symbol is a entry point,
    * n_value will be 0. However, one thing is for sure 
-   * that if a symbol is not existance in the file,
+   * that if a symbol does not exists in the file,
    * both .n_type and .n_value would be 0.
    */
 
   if (!nl[1].n_type && !nl[1].n_value)
     {
-      errmsg = "symbol not existance in this module";
+      errmsg = "symbol does not exist in this module";
       return 0;
     }
 
@@ -528,7 +575,7 @@ dlsym (void *hdl, char *sym)
    * by adding a relative offset getting from the module 
    * file symbol table, i.e
    *
-   *  functin virtual address = entry point virtual address +
+   *  function virtual address = entry point virtual address +
    *     + ( function offset in file - entry point offset in file )
    */
 
@@ -552,8 +599,8 @@ dlsym (void *hdl, char *sym)
 
 #include <windows.h>
 
-void FAR *
-dlopen (char FAR * dll, int mode)
+void *
+dlopen (char * dll, int mode)
 {
   HINSTANCE hint;
 
@@ -569,18 +616,18 @@ dlopen (char FAR * dll, int mode)
       return NULL;
     }
 
-  return (void FAR *) hint;
+  return (void *) hint;
 }
 
 
-void FAR *
-dlsym (void FAR * hdll, char FAR * sym)
+void *
+dlsym (void * hdll, char * sym)
 {
-  return (void FAR *) GetProcAddress (hdll, sym);
+  return (void *) GetProcAddress (hdll, sym);
 }
 
 
-char FAR *
+char *
 dlerror ()
 {
   return 0L;			/* unimplemented yet */
@@ -588,7 +635,7 @@ dlerror ()
 
 
 int
-dlclose (void FAR * hdll)
+dlclose (void * hdll)
 {
   FreeLibrary ((HINSTANCE) hdll);
 }
@@ -673,7 +720,7 @@ iodbc_dlopen (char *path, int unused_flag)
 
   imgfab = cc$rms_fab;
   imgfab.fab$l_fna = path;
-  imgfab.fab$b_fns = strlen (path);
+  imgfab.fab$b_fns = STRLEN (path);
   imgfab.fab$w_ifi = 0;
   imgfab.fab$l_dna = defimg;
   imgfab.fab$b_dns = sizeof (defimg);
@@ -738,7 +785,7 @@ iodbc_dlsym (void *hdll, char *sym)
   symbol_d.dsc$b_dtype = DSC$K_DTYPE_T;
   symbol_d.dsc$b_class = DSC$K_CLASS_S;
   symbol_d.dsc$a_pointer = sym;
-  symbol_d.dsc$w_length = strlen (sym);
+  symbol_d.dsc$w_length = STRLEN (sym);
   status = iodbc_find_image_symbol (&dll->filename_d, &symbol_d, &rp,
       &dll->image_d, 0);
   if (!((saved_status ^ LIB$_KEYNOTFOU) & ~7))
@@ -878,7 +925,7 @@ dlopen (char *path, int mode)
       {
 	  "%s(%d): Object Image Load Failure\n",
 	  "%s(%d): Object Image Load Success\n",
-	  "%s(%d): Not an recognisable object file\n",
+	  "%s(%d): Not an recognizable object file\n",
 	  "%s(%d): No valid architecture\n",
 	  "%s(%d): Object image has an invalid format\n",
 	  "%s(%d): Invalid access (permissions?)\n",
@@ -954,13 +1001,13 @@ static char *dlerror_pointer = NULL;
 /*
  * NSMakePrivateModulePublic() is not part of the public dyld API so we define
  * it here.  The internal dyld function pointer for
- * __dyld_NSMakePrivateModulePublic is returned so thats all that maters to get
+ * __dyld_NSMakePrivateModulePublic is returned so thats all that matters to get
  * the functionality need to implement the dlopen() interfaces.
  */
-static enum bool
+static int
 NSMakePrivateModulePublic (NSModule module)
 {
-  static enum bool (*p) (NSModule module) = NULL;
+  static int (*p) (NSModule module) = NULL;
 
   if (p == NULL)
     _dyld_func_lookup ("__dyld_NSMakePrivateModulePublic",
@@ -980,8 +1027,8 @@ NSMakePrivateModulePublic (NSModule module)
 /*
  * dlopen() the MacOS X version of the FreeBSD dlopen() interface.
  */
-void FAR *
-dlopen (char FAR * path, int mode)
+void *
+iodbc_dlopen (char * path, int mode)
 {
   void *retval;
   struct stat stat_buf;
@@ -992,8 +1039,10 @@ dlopen (char FAR * path, int mode)
   unsigned long options;
   NSSymbol NSSymbol;
   void (*init) (void);
+  static char errbuf[640];
 
   dlerror_pointer = NULL;
+
   /*
    * A NULL path is to indicate the caller wants a handle for the
    * main program.
@@ -1096,14 +1145,23 @@ dlopen (char FAR * path, int mode)
     }
 
   /* try to link in this object file image */
-  options = NSLINKMODULE_OPTION_PRIVATE;
+  options = NSLINKMODULE_OPTION_NONE |
+    NSLINKMODULE_OPTION_PRIVATE |
+    NSLINKMODULE_OPTION_RETURN_ON_ERROR;
   if ((mode & RTLD_NOW) == RTLD_NOW)
     options |= NSLINKMODULE_OPTION_BINDNOW;
   module = NSLinkModule (objectFileImage, path, options);
   NSDestroyObjectFileImage (objectFileImage);
   if (module == NULL)
     {
-      dlerror_pointer = "NSLinkModule() failed for dlopen()";
+      NSLinkEditErrors lerr;
+      int errNum;
+      const char *fname;
+      const char *errStr;
+      NSLinkEditError(&lerr, &errNum, &fname, &errStr);
+      sprintf(errbuf, "NSLinkModule() failed for dlopen() ([%.256s][%.256s])",
+      		fname, errStr);
+      dlerror_pointer = errbuf;
       return (NULL);
     }
 
@@ -1160,8 +1218,8 @@ dlopen (char FAR * path, int mode)
 /*
  * dlsym() the MacOS X version of the FreeBSD dlopen() interface.
  */
-void FAR *
-dlsym (void FAR * handle, char FAR * symbol)
+void *
+iodbc_dlsym (void * handle, char * symbol)
 {
   struct dlopen_handle *dlopen_handle, *p;
   char symbol2[1024];
@@ -1224,14 +1282,14 @@ dlsym (void FAR * handle, char FAR * symbol)
 /*
  * dlerror() the MacOS X version of the FreeBSD dlopen() interface.
  */
-char FAR *
-dlerror (void)
+char *
+iodbc_dlerror (void)
 {
   const char *p;
 
   p = (const char *) dlerror_pointer;
   dlerror_pointer = NULL;
-  return (p);
+  return (char *)(p);
 }
 
 
@@ -1239,7 +1297,7 @@ dlerror (void)
  * dlclose() the MacOS X version of the FreeBSD dlopen() interface.
  */
 int
-dlclose (void FAR * handle)
+iodbc_dlclose (void * handle)
 {
   struct dlopen_handle *p, *q;
   unsigned long options;
@@ -1305,7 +1363,7 @@ dlclose (void FAR * handle)
 #include <CodeFragments.h>
 #include <strconv.h>
 
-static OSErr error = noErr;
+static char *msg_error = NULL;
 
 void *
 dlopen (char *dll, int mode)
@@ -1314,23 +1372,46 @@ dlopen (char *dll, int mode)
   CFragConnectionID conn_id;
   Ptr main_addr;
   Str255 name;
+  OSErr err;
 
   if (dll == NULL)
     {
+      msg_error = "Library name not valid.";
       return NULL;
     }
 
-  error =
-      GetSharedLibrary ((unsigned char *) str_to_Str255 (dll),
-      kPowerPCCFragArch, kLoadCFrag, &conn_id, &main_addr, name);
-
-  if (error != noErr)
+  if ((err = GetSharedLibrary ((unsigned char *) str_to_Str255 (dll),
+	      kPowerPCCFragArch, kLoadCFrag, &conn_id, &main_addr,
+	      name)) != noErr)
     {
+      msg_error = "Library cannot be loaded.";
       return NULL;
     }
 
+  msg_error = NULL;
   return (void *) conn_id;
 #else
+  CFragConnectionID conn_id;
+  Ptr main_addr;
+  Str255 name;
+  OSErr err;
+
+  if (dll == NULL)
+    {
+      msg_error = "Library name not valid.";
+      return NULL;
+    }
+
+  if ((err = GetSharedLibrary ((unsigned char *) str_to_Str255 (dll),
+	      kMotorola68KCFragArch, kLoadCFrag, &conn_id, &main_addr,
+	      name)) != noErr)
+    {
+      msg_error = "Library cannot be loaded.";
+      return NULL;
+    }
+
+  msg_error = NULL;
+  return (void *) conn_id;
 #endif
 }
 
@@ -1341,18 +1422,45 @@ dlsym (void *hdll, char *sym)
 #ifdef __POWERPC__
   Ptr symbol;
   CFragSymbolClass symbol_type;
+  OSErr err;
 
-  error =
-      FindSymbol ((CFragConnectionID) hdll,
-      (unsigned char *) str_to_Str255 (sym), &symbol, &symbol_type);
-
-  if (error != noErr)
+  if (sym == NULL)
     {
+      msg_error = "Symbol name not valid.";
       return NULL;
     }
 
+  if ((err =
+	  FindSymbol ((CFragConnectionID) hdll,
+	   (unsigned char *) str_to_Str255 (sym), &symbol,
+	      &symbol_type)) != noErr)
+    {
+      msg_error = "Symbol cannot be loaded.";
+      return NULL;
+    }
+
+  msg_error = NULL;
   return symbol;
 #else
+  Ptr symbol;
+  CFragSymbolClass symbol_type;
+
+  if (sym == NULL)
+    {
+      msg_error = "Symbol name not valid.";
+      return NULL;
+    }
+
+  if (FindSymbol ((CFragConnectionID) hdll,
+	  (unsigned char *) str_to_Str255 (sym), &symbol,
+	  &symbol_type) != noErr)
+    {
+      msg_error = "Symbol cannot be loaded.";
+      return NULL;
+    }
+
+  msg_error = NULL;
+  return symbol;
 #endif
 }
 
@@ -1360,18 +1468,35 @@ dlsym (void *hdll, char *sym)
 char *
 dlerror ()
 {
-  return 0L;			/* unimplemented yet */
+  return (msg_error) ? msg_error : "No error detected.";
 }
 
 
 int
 dlclose (void *hdll)
 {
+  /* It should be something like that ....  */
+  /* but some applications like Office 2001 */
+  /* have a problem with that ... just let  */
+  /* the Mac unload the library when the    */
+  /* application stop ...                   */
 #ifdef __POWERPC__
-  error = CloseConnection ((CFragConnectionID *) hdll);
-
-  return error;
+/*		if( CloseConnection((CFragConnectionID*)hdll) )
+		{
+			msg_error = "Library cannot be unloaded.";
+			return 1;
+		}
+		msg_error = NULL;	
+		return 0;*/
 #else
+  if (CloseConnection ((CFragConnectionID *) hdll))
+    {
+      msg_error = "Library cannot be unloaded.";
+      return 1;
+    }
+
+  msg_error = NULL;
+  return 0;
 #endif
 }
 
