@@ -1,7 +1,7 @@
 /*
  *  henv.c
  *
- *  $Id: henv.c,v 1.17 2005/02/01 14:22:09 source Exp $
+ *  $Id: henv.c,v 1.19 2005/05/12 12:27:15 source Exp $
  *
  *  Environment object management functions
  *
@@ -76,7 +76,7 @@
 #include <sql.h>
 #include <sqlext.h>
 
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include <dlproc.h>
 
@@ -281,7 +281,7 @@ Init_iODBC (void)
 }
 
 
-static void 
+void 
 Done_iODBC(void)
 {
 #if !defined (PTHREAD_MUTEX_INITIALIZER) || defined (WINDOWS)
