@@ -1,7 +1,7 @@
 /*
  *  hstmt.h
  *
- *  $Id: hstmt.h,v 1.13 2003/08/22 14:28:41 source Exp $
+ *  $Id: hstmt.h,v 1.17 2004/08/26 12:12:54 source Exp $
  *
  *  Query statement object management functions
  *
@@ -91,7 +91,7 @@ typedef struct BIND {
   SWORD		 bn_type;	  /* ODBC C data type */
   void *	 bn_data;	  /* Pointer to data */
   SDWORD	 bn_size;	  /* Size of data area */
-  SDWORD	*bn_pInd;	  /* Holds SQL_NULL_DATA | 0. 
+  SQLLEN	*bn_pInd;	  /* Holds SQL_NULL_DATA | 0. 
                                    * And length of returned char/bin data 
 				   */
 } BIND_t;
@@ -130,8 +130,8 @@ typedef struct STMT
     SQLUINTEGER bind_type;
 
 #if (ODBCVER >= 0x0300)
-    DESC_t FAR * imp_desc[4];
-    DESC_t FAR * desc[4];
+    DESC_t * imp_desc[4];
+    DESC_t * desc[4];
     SQLUINTEGER row_array_size;
     SQLPOINTER fetch_bookmark_ptr, params_processed_ptr;
     SQLUINTEGER paramset_size;
@@ -152,8 +152,8 @@ STMT_t;
 
 #define IS_VALID_HSTMT(x) \
 	((x) != SQL_NULL_HSTMT && \
-	 ((STMT_t FAR *)(x))->type == SQL_HANDLE_STMT && \
-	 ((STMT_t FAR *)(x))->hdbc != SQL_NULL_HDBC)
+	 ((STMT_t *)(x))->type == SQL_HANDLE_STMT && \
+	 ((STMT_t *)(x))->hdbc != SQL_NULL_HDBC)
 
 
 #define ENTER_STMT(hstmt, trace) \
@@ -176,7 +176,7 @@ STMT_t;
 	CLEAR_ERRORS (pstmt); \
 	if (pstmt->asyn_on == en_NullProc && pstmt->params_inserted > 0) \
 	  _iodbcdm_FreeStmtParams(pstmt); \
-        ODBC_UNLOCK();
+        ODBC_UNLOCK()
 	
 
 #define LEAVE_STMT(hstmt, trace) \
@@ -185,7 +185,7 @@ STMT_t;
     done: \
     	TRACE(trace); \
 	ODBC_UNLOCK (); \
-	return (retcode);
+	return (retcode)
 
 
 enum
@@ -215,37 +215,37 @@ enum
 /*
  *  Internal prototypes
  */
-SQLRETURN _iodbcdm_dropstmt ();
+SQLRETURN _iodbcdm_dropstmt (HSTMT stmt);
 
 void _iodbcdm_FreeStmtParams(STMT_t *pstmt);
 void *_iodbcdm_alloc_param(STMT_t *pstmt, int i, int size);
-wchar_t *_iodbcdm_conv_param_A2W(STMT_t *pstmt, int i, char *pData, int pDataLength);
-char *_iodbcdm_conv_param_W2A(STMT_t *pstmt, int i, wchar_t *pData, int pDataLength);
+wchar_t *_iodbcdm_conv_param_A2W(STMT_t *pstmt, int i, SQLCHAR *pData, int pDataLength);
+char *_iodbcdm_conv_param_W2A(STMT_t *pstmt, int i, SQLWCHAR *pData, int pDataLength);
 void _iodbcdm_ConvBindData (STMT_t *pstmt);
 SQLRETURN _iodbcdm_BindColumn (STMT_t *pstmt, BIND_t *pbind);
 int _iodbcdm_UnBindColumn (STMT_t *pstmt, BIND_t *pbind);
 void _iodbcdm_RemoveBind (STMT_t *pstmt);
-void _iodbcdm_do_cursoropen (STMT_t FAR * pstmt);
+void _iodbcdm_do_cursoropen (STMT_t * pstmt);
 SQLSMALLINT _iodbcdm_map_sql_type (int type, int odbcver);
 SQLSMALLINT _iodbcdm_map_c_type (int type, int odbcver);
 
 
 SQLRETURN SQL_API _iodbcdm_ExtendedFetch (
-    SQLHSTMT hstmt, 
-    SQLUSMALLINT fFetchType, 
-    SQLINTEGER irow, 
-    SQLUINTEGER FAR *pcrow, 
-    SQLUSMALLINT FAR *rgfRowStatus);
+    SQLHSTMT		  hstmt,
+    SQLUSMALLINT	  fFetchType,
+    SQLLEN		  irow, 
+    SQLULEN	 	* pcrow, 
+    SQLUSMALLINT 	* rgfRowStatus);
 
 SQLRETURN SQL_API _iodbcdm_SetPos (
-    SQLHSTMT hstmt, 
-    SQLUSMALLINT irow, 
-    SQLUSMALLINT fOption, 
-    SQLUSMALLINT fLock);
+    SQLHSTMT		  hstmt, 
+    SQLSETPOSIROW	  irow, 
+    SQLUSMALLINT	  fOption, 
+    SQLUSMALLINT	  fLock);
 
 SQLRETURN SQL_API _iodbcdm_NumResultCols (
     SQLHSTMT hstmt,
-    SQLSMALLINT FAR * pccol);
+    SQLSMALLINT * pccol);
 
 SQLRETURN SQLGetStmtOption_Internal (
   SQLHSTMT	hstmt, 
