@@ -1,7 +1,7 @@
 /*
  *  info.c
  *
- *  $Id: info.c,v 1.29 2004/08/16 11:04:05 source Exp $
+ *  $Id: info.c,v 1.30 2005/05/12 12:27:15 source Exp $
  *
  *  Information functions
  *
@@ -76,7 +76,7 @@
 #include <sql.h>
 #include <sqlext.h>
 #include <sqlucode.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include <unicode.h>
 
