@@ -1,7 +1,7 @@
 /*
  *  misc.c
  *
- *  $Id: misc.c,v 1.20 2004/06/21 13:34:29 source Exp $
+ *  $Id: misc.c,v 1.21 2005/05/12 12:27:15 source Exp $
  *
  *  Miscellaneous functions
  *
@@ -75,7 +75,7 @@
 
 #include <sql.h>
 #include <sqlext.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include <stdio.h>
 #include <stdlib.h>
