@@ -1,7 +1,7 @@
 /*
  *  DriverConnect.c
  *
- *  $Id: DriverConnect.c,v 1.3 2004/06/21 13:32:47 source Exp $
+ *  $Id: DriverConnect.c,v 1.4 2004/11/03 15:22:43 source Exp $
  *
  *  SQLDriverConnect trace functions
  *
@@ -94,21 +94,21 @@ _trace_connstr_hidepwd (SQLCHAR *str)
 	  break;
 
 	case 0:
-	  if (*ptr == 'P')
+	  if (toupper(*ptr) == 'P')
 	    state = 1;
 	  else if (strchr ("\'\"{", *ptr))
 	    state = -1;		/* in string */
 	  break;
 
 	case 1:
-	  if (*ptr == 'W')
+	  if (toupper(*ptr) == 'W')
 	    state = 2;
 	  else
 	    state = 0;
 	  break;
 
 	case 2:
-	  if (*ptr == 'D')
+	  if (toupper(*ptr) == 'D')
 	    state = 3;
 	  else
 	    state = 0;
