@@ -1,7 +1,7 @@
 /*
  *  trace.c
  *
- *  $Id: trace.c,v 1.12 2005/02/01 01:11:43 source Exp $
+ *  $Id: trace.c,v 1.15 2005/10/08 15:13:03 source Exp $
  *
  *  Trace functions
  *
@@ -69,15 +69,24 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <unistd.h>
-#include <time.h>
 #include <fcntl.h>
 
+#include <time.h>
+#ifdef TIME_WITH_SYS_TIME
+#include <sys/time.h>
+#endif
+
 #include <sql.h>
 #include <sqlext.h>
 #include <sqlucode.h>
@@ -110,6 +119,10 @@ static int   trace_fp_close = 0;
 
 void trace_emit (char *fmt, ...);
 
+#ifdef HAVE_GETTIMEOFDAY
+static struct timeval starttime = {0};
+#endif
+
 
 /*
  * Internal functions
@@ -263,6 +276,10 @@ trace_start(void)
    */
   trace_stop ();
 
+#ifdef HAVE_GETTIMEOFDAY
+  gettimeofday (&starttime, NULL);
+#endif
+
   /*
    *  If no trace filename is specified, use the default
    */
@@ -367,6 +384,16 @@ trace_start(void)
     else
       trace_set_appname ("{No Application Name}");
   }
+#elif defined(__APPLE__)
+#ifdef MACOSX102
+  {
+    trace_set_appname ("{No Application Name}");
+  }
+#else
+  {
+    trace_set_appname (getprogname());
+  }
+#endif
 #endif
 
   /*
@@ -624,6 +651,24 @@ _trace_print_function (int func, int trace_leave, int retcode)
 {
   extern char *odbcapi_symtab[];
   char *ptr = "invalid retcode";
+  struct timeval tv;
+
+  /*
+   * Calculate timestamp
+   */
+#ifdef HAVE_GETTIMEOFDAY
+  gettimeofday (&tv, NULL);
+  tv.tv_sec -= starttime.tv_sec;
+  tv.tv_usec -= starttime.tv_usec;
+  if (tv.tv_usec < 0)
+    {
+      tv.tv_sec--;
+      tv.tv_usec += 1000000L;
+    }
+  trace_emit ("\n[%06ld.%06ld]\n", tv.tv_sec, tv.tv_usec);
+#else
+  trace_emit ("\n");
+#endif
 
   switch (retcode)
     {
@@ -641,11 +686,11 @@ _trace_print_function (int func, int trace_leave, int retcode)
 #endif
 
   if (trace_leave == TRACE_LEAVE)
-    trace_emit ("\n%-15.15s %08lX EXIT  %s with return code %d (%s)\n",
+    trace_emit ("%-15.15s %08lX EXIT  %s with return code %d (%s)\n",
 	trace_appname ? trace_appname : "Application",
 	THREAD_IDENT, odbcapi_symtab[func], retcode, ptr);
   else
-    trace_emit ("\n%-15.15s %08lX ENTER %s\n",
+    trace_emit ("%-15.15s %08lX ENTER %s\n",
 	trace_appname ? trace_appname : "Application",
 	THREAD_IDENT, odbcapi_symtab[func]);
 }
