@@ -1,7 +1,7 @@
 /*
  *  error.c
  *
- *  $Id: error.c,v 1.4 2004/06/22 11:23:47 source Exp $
+ *  $Id: error.c,v 1.5 2005/03/02 17:17:59 source Exp $
  *
  *  The data_sources dialog for SQLDriverConnect and a login box procedures
  *
@@ -119,6 +119,19 @@ _iodbcdm_errorbox (
 }
 
 
+void SQL_API
+_iodbcdm_errorboxw (
+    HWND hwnd,
+    LPCWSTR szDSN,
+    LPCWSTR szText)
+{
+  wchar_t msg[4096];
+
+  if (SQLInstallerErrorW (1, NULL, msg, sizeof (msg) / sizeof(wchar_t), NULL) == SQL_SUCCESS)
+    create_errorw (hwnd, szDSN, szText, msg);
+}
+
+
 void SQL_API
 _iodbcdm_messagebox (
     HWND	hwnd,
@@ -129,6 +142,16 @@ _iodbcdm_messagebox (
 }
 
 
+void SQL_API
+_iodbcdm_messageboxw (
+    HWND hwnd,
+    LPCWSTR szDSN,
+    LPCWSTR szText)
+{
+  create_messagew (hwnd, szDSN, szText);
+}
+
+
 BOOL SQL_API
 _iodbcdm_confirmbox (
     HWND	hwnd,
@@ -137,3 +160,14 @@ _iodbcdm_confirmbox (
 {
   return create_confirm (hwnd, (SQLPOINTER) szDSN, (SQLPOINTER) szText);
 }
+
+
+BOOL SQL_API
+_iodbcdm_confirmboxw (
+    HWND hwnd,
+	 LPCWSTR szDSN,
+	 LPCWSTR szText)
+{
+  return create_confirmw (hwnd, (SQLPOINTER)szDSN, (SQLPOINTER)szText);
+}
+
