@@ -1,7 +1,7 @@
 /*
  *  administrator.c
  *
- *  $Id: administrator.c,v 1.8 2004/06/22 11:23:47 source Exp $
+ *  $Id: administrator.c,v 1.10 2005/03/14 10:35:43 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -579,7 +579,7 @@ driver_add_clicked (GtkWidget *widget, TDRIVERCHOOSER *choose_t)
 
   if (choose_t)
     {
-      cstr = create_driversetup (choose_t->mainwnd, NULL, connstr, FALSE);
+      cstr = create_driversetup (choose_t->mainwnd, NULL, connstr, FALSE, TRUE);
 
       if (cstr && cstr != connstr && cstr != (LPSTR) - 1L)
 	{
@@ -700,7 +700,7 @@ driver_configure_clicked (GtkWidget *widget, TDRIVERCHOOSER *choose_t)
 
 	  cstr =
 	      create_driversetup (choose_t->mainwnd, szDriver, connstr,
-	      FALSE);
+	      FALSE, TRUE);
 
 	  if (cstr && cstr != connstr && cstr != (LPSTR) - 1L)
 	    {
@@ -756,7 +756,8 @@ admin_ok_clicked (GtkWidget *widget, void **inparams)
       dsnchoose_t->file_list = dsnchoose_t->file_entry =
 	  dsnchoose_t->dir_combo = NULL;
       dsnchoose_t->type_dsn = -1;
-      dsnchoose_t->curr_dir = dsnchoose_t->dsn = NULL;
+      dsnchoose_t->curr_dir = NULL;
+      dsnchoose_t->dsn = NULL;
     }
 
   if (driverchoose_t)
@@ -828,7 +829,8 @@ admin_cancel_clicked (GtkWidget *widget, void **inparams)
       dsnchoose_t->file_list = dsnchoose_t->file_entry =
 	  dsnchoose_t->dir_combo = NULL;
       dsnchoose_t->type_dsn = -1;
-      dsnchoose_t->curr_dir = dsnchoose_t->dsn = NULL;
+      dsnchoose_t->curr_dir = NULL;
+      dsnchoose_t->dsn = NULL;
     }
 
   if (driverchoose_t)
