@@ -1,7 +1,7 @@
 /*
  *  driverchooser.c
  *
- *  $Id: driverchooser.c,v 1.6 2004/10/23 19:38:54 source Exp $
+ *  $Id: driverchooser.c,v 1.7 2005/03/14 10:35:43 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -323,7 +323,7 @@ driverchooser_ok_clicked (GtkWidget *widget, TDRIVERCHOOSER *choose_t)
 	  gtk_clist_get_text (GTK_CLIST (choose_t->driverlist),
 	      GPOINTER_TO_INT (GTK_CLIST (choose_t->driverlist)->selection->
 		  data), 0, &szDriver);
-	  choose_t->driver = strdup (szDriver);
+	  choose_t->driver = dm_SQL_A2W(szDriver, SQL_NTS);
 	}
       else
 	choose_t->driver = NULL;
