@@ -1,7 +1,7 @@
 /*
  *  messagebox.c
  *
- *  $Id: messagebox.c,v 1.3 2004/06/22 11:23:47 source Exp $
+ *  $Id: messagebox.c,v 1.5 2005/03/14 10:35:43 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -183,3 +183,20 @@ create_message (HWND hwnd, LPCSTR dsn, LPCSTR text)
   gtk_widget_show_all (message);
   gtk_main ();
 }
+
+void
+create_messagew (HWND hwnd, LPCWSTR dsn, LPCWSTR text)
+{
+  LPSTR _dsn = NULL;
+  LPSTR _text = NULL;
+
+  _dsn = dm_SQL_WtoU8(dsn, SQL_NTS);
+  _text = dm_SQL_WtoU8(text, SQL_NTS);
+
+  create_message(hwnd, _dsn, _text);
+
+  if (_dsn)
+    free(_dsn);
+  if (_text)
+    free(_text);
+}
