@@ -1,7 +1,7 @@
 /*
  *  iodbcadm.h
  *
- *  $Id: iodbcadm.h,v 1.6 2004/06/22 11:23:47 source Exp $
+ *  $Id: iodbcadm.h,v 1.9 2005/08/15 21:04:15 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -68,12 +68,11 @@
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifdef _MACX
-#  include <iODBCinst/iodbcinst.h>
-#else
-#  include <iodbc.h>
-#  include <iodbcinst.h>
-#endif
+#include <iodbc.h>
+#include <odbcinst.h>
+#include <sql.h>
+#include <sqltypes.h>
+#include <sqlucode.h>
 
 #ifndef	_IODBCADM_H
 #define	_IODBCADM_H
@@ -91,12 +90,20 @@ SQLRETURN SQL_API iodbcdm_drvconn_dialboxw (HWND hwnd, LPWSTR szInOutConnStr,
 
 SQLRETURN SQL_API _iodbcdm_drvchoose_dialbox (HWND hwnd, LPSTR szInOutDrvStr,
     DWORD cbInOutDrvStr, int *sqlStat);
+SQLRETURN SQL_API _iodbcdm_drvchoose_dialboxw (HWND hwnd, LPWSTR szInOutConnStr,
+    DWORD cbInOutConnStr, int FAR * sqlStat);
+
 SQLRETURN SQL_API _iodbcdm_trschoose_dialbox (HWND hwnd, LPSTR szInOutDrvStr,
     DWORD cbInOutDrvStr, int *sqlStat);
+SQLRETURN SQL_API _iodbcdm_trschoose_dialboxw (HWND hwnd, LPWSTR szInOutDrvStr,
+    DWORD cbInOutDrvStr, int FAR * sqlStat);
 
 void SQL_API _iodbcdm_errorbox (HWND hwnd, LPCSTR szDSN, LPCSTR szText);
+void SQL_API _iodbcdm_errorboxw (HWND hwnd, LPCWSTR szDSN, LPCWSTR szText);
 void SQL_API _iodbcdm_messagebox (HWND hwnd, LPCSTR szDSN, LPCSTR szText);
+void SQL_API _iodbcdm_messageboxw (HWND hwnd, LPCWSTR szDSN, LPCWSTR szText);
 BOOL SQL_API _iodbcdm_confirmbox (HWND hwnd, LPCSTR szDSN, LPCSTR szText);
+BOOL SQL_API _iodbcdm_confirmboxw (HWND hwnd, LPCWSTR szDSN, LPCWSTR szText);
 void _iodbcdm_nativeerrorbox (HWND hwnd, HENV henv, HDBC hdbc, HSTMT hstmt);
 
 SQLRETURN SQL_API _iodbcdm_admin_dialbox (HWND hwnd);
@@ -107,4 +114,7 @@ typedef SQLRETURN SQL_API (*pTrsChooseFunc) (HWND hwnd, LPSTR szInOutDrvStr,
 typedef SQLRETURN SQL_API (*pDrvConnFunc) (HWND hwnd, LPSTR szInOutConnStr,
     DWORD cbInOutConnStr, int *sqlStat, SQLUSMALLINT fDriverCompletion,
     UWORD * config);
+typedef SQLRETURN SQL_API (*pDrvConnWFunc) (HWND hwnd, LPWSTR szInOutConnStr,
+    DWORD cbInOutConnStr, int FAR * sqlStat, SQLUSMALLINT *fDriverCompletion, 
+    UWORD *config);
 #endif
