@@ -1,7 +1,7 @@
 /*
  *  SQLGetAvailableDrivers.c
  *
- *  $Id: SQLGetAvailableDrivers.c,v 1.5 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLGetAvailableDrivers.c,v 1.9 2005/09/12 08:28:43 source Exp $
  *
  *  Get a list of all available drivers
  *
@@ -71,16 +71,16 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "misc.h"
 #include "inifile.h"
 #include "iodbc_error.h"
 
-
 BOOL
 GetAvailableDrivers (LPCSTR lpszInfFile, LPSTR lpszBuf, WORD cbBufMax,
-    WORD *pcbBufOut, BOOL infFile)
+    WORD * pcbBufOut, BOOL infFile)
 {
   int sect_len = 0;
   WORD curr = 0;
@@ -164,7 +164,7 @@ quit:
 
 BOOL INSTAPI
 SQLGetAvailableDrivers (LPCSTR lpszInfFile, LPSTR lpszBuf, WORD cbBufMax,
-    WORD *pcbBufOut)
+    WORD * pcbBufOut)
 {
   BOOL retcode = FALSE;
   WORD lenBufOut;
@@ -182,7 +182,7 @@ SQLGetAvailableDrivers (LPCSTR lpszInfFile, LPSTR lpszBuf, WORD cbBufMax,
     case ODBC_SYSTEM_DSN:
       wSystemDSN = SYSTEMDSN_ONLY;
       break;
-    };
+    }
 
   retcode =
       GetAvailableDrivers (lpszInfFile, lpszBuf, cbBufMax, &lenBufOut, FALSE);
@@ -194,3 +194,57 @@ SQLGetAvailableDrivers (LPCSTR lpszInfFile, LPSTR lpszBuf, WORD cbBufMax,
   configMode = ODBC_BOTH_DSN;
   return retcode;
 }
+
+BOOL INSTAPI
+SQLGetAvailableDriversW (LPCWSTR lpszInfFile, LPWSTR lpszBuf, WORD cbBufMax,
+    WORD FAR * pcbBufOut)
+{
+  BOOL retcode = FALSE;
+  char *_inf_u8 = NULL;
+  char *_buffer_u8 = NULL;
+  SQLCHAR *ptr;
+  SQLWCHAR *ptrW;
+  WORD len = 0, length;
+
+  _inf_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszInfFile, SQL_NTS);
+  if (_inf_u8 == NULL && lpszInfFile)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  if (cbBufMax > 0)
+    {
+      if ((_buffer_u8 = malloc (cbBufMax * UTF8_MAX_CHAR_LEN + 1)) == NULL)
+	{
+	  PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+	  goto done;
+	}
+    }
+
+  retcode =
+      SQLGetAvailableDrivers (_inf_u8, _buffer_u8,
+      cbBufMax * UTF8_MAX_CHAR_LEN, pcbBufOut);
+
+  if (retcode == TRUE)
+    {
+      length = 0;
+
+      for (ptr = _buffer_u8, ptrW = lpszBuf; *ptr;
+	  ptr += STRLEN (ptr) + 1, ptrW += WCSLEN (ptrW) + 1)
+	{
+	  dm_StrCopyOut2_U8toW (ptr, ptrW, cbBufMax - 1, &len);
+	  length += len;
+	}
+
+      *ptrW = L'\0';
+      if (pcbBufOut)
+	*pcbBufOut = length + 1;
+    }
+
+done:
+  MEM_FREE (_inf_u8);
+  MEM_FREE (_buffer_u8);
+
+  return retcode;
+}
