@@ -1,7 +1,7 @@
 /*
  *  SQLGetConfigMode.c
  *
- *  $Id: SQLGetConfigMode.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLGetConfigMode.c,v 1.5 2005/05/12 12:27:15 source Exp $
  *
  *  These functions intentionally left blank
  *
@@ -71,14 +71,14 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include "iodbc_error.h"
 #include "misc.h"
 
 
 BOOL INSTAPI
-SQLGetConfigMode (UWORD *pwConfigMode)
+SQLGetConfigMode (UWORD * pwConfigMode)
 {
   BOOL retcode = FALSE;
 
