@@ -1,7 +1,7 @@
 /*
  *  SQLGetProfileString.c
  *
- *  $Id: SQLGetPrivateProfileString.c,v 1.4 2004/07/06 15:01:32 source Exp $
+ *  $Id: SQLGetPrivateProfileString.c,v 1.9 2005/09/12 08:28:43 source Exp $
  *
  *  The iODBC driver manager.
  *
@@ -69,13 +69,13 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "inifile.h"
 #include "iodbc_error.h"
 #include "misc.h"
 
-
 #ifndef WIN32
 int
 GetPrivateProfileString (LPCSTR lpszSection, LPCSTR lpszEntry,
@@ -91,6 +91,8 @@ GetPrivateProfileString (LPCSTR lpszSection, LPCSTR lpszEntry,
   /* If error during reading the file */
   if (_iodbcdm_cfg_search_init (&pCfg, lpszFilename, FALSE))
     {
+      if (lpszDefault)
+	STRNCPY (lpszRetBuffer, lpszDefault, cbRetBuffer - 1);
       PUSH_ERROR (ODBC_ERROR_INVALID_PATH);
       goto fail;
     }
@@ -225,20 +227,23 @@ SQLGetPrivateProfileString (LPCSTR lpszSection, LPCSTR lpszEntry,
 	  goto quit;
 	}
       if (_iodbcadm_getinifile (pathbuf, sizeof (pathbuf), FALSE, FALSE))
-	len =
+        {
+	  len =
 	    GetPrivateProfileString (lpszSection, lpszEntry, lpszDefault,
 	    lpszRetBuffer, cbRetBuffer, pathbuf);
-      else
-	{
-	  CLEAR_ERROR ();
-	  wSystemDSN = SYSTEMDSN_ONLY;
-	  if (_iodbcadm_getinifile (pathbuf, sizeof (pathbuf), FALSE, FALSE))
-	    len =
-		GetPrivateProfileString (lpszSection, lpszEntry, lpszDefault,
+          if (len)
+            goto quit;
+        }
+      CLEAR_ERROR ();
+      wSystemDSN = SYSTEMDSN_ONLY;
+      if (_iodbcadm_getinifile (pathbuf, sizeof (pathbuf), FALSE, FALSE))
+        {
+          len =
+  	    GetPrivateProfileString (lpszSection, lpszEntry, lpszDefault,
 		lpszRetBuffer, cbRetBuffer, pathbuf);
 	}
       goto quit;
-    };
+    }
 
   PUSH_ERROR (ODBC_ERROR_GENERAL_ERR);
   goto quit;
@@ -248,3 +253,124 @@ quit:
   configMode = ODBC_BOTH_DSN;
   return len;
 }
+
+int INSTAPI
+SQLGetPrivateProfileStringW (LPCWSTR lpszSection, LPCWSTR lpszEntry,
+    LPCWSTR lpszDefault, LPWSTR lpszRetBuffer, int cbRetBuffer,
+    LPCWSTR lpszFilename)
+{
+  char *_section_u8 = NULL;
+  char *_entry_u8 = NULL;
+  char *_default_u8 = NULL;
+  char *_buffer_u8 = NULL;
+  char *_filename_u8 = NULL;
+  SQLCHAR *ptr;
+  SQLWCHAR *ptrW;
+  SQLSMALLINT length, len;
+
+  _section_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszSection, SQL_NTS);
+  if (_section_u8 == NULL && lpszSection)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _entry_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszEntry, SQL_NTS);
+  if (_entry_u8 == NULL && lpszEntry)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _default_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszDefault, SQL_NTS);
+  if (_default_u8 == NULL && lpszDefault)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _filename_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszFilename, SQL_NTS);
+  if (_filename_u8 == NULL && lpszFilename)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  if (cbRetBuffer > 0)
+    {
+      if ((_buffer_u8 = malloc (cbRetBuffer * UTF8_MAX_CHAR_LEN + 1)) == NULL)
+	{
+	  PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+	  goto done;
+	}
+    }
+
+  length = SQLGetPrivateProfileString (_section_u8, _entry_u8, _default_u8,
+      _buffer_u8, cbRetBuffer * UTF8_MAX_CHAR_LEN, _filename_u8);
+
+  if (length > 0)
+    {
+      if (lpszSection == NULL || lpszEntry == NULL ||
+	  lpszSection[0] == '\0' || lpszEntry[0] == '\0')
+	{
+	  length = 0;
+
+	  for (ptr = _buffer_u8, ptrW = lpszRetBuffer; *ptr;
+	      ptr += STRLEN (ptr) + 1, ptrW += WCSLEN (ptrW) + 1)
+	    {
+	      dm_StrCopyOut2_U8toW (ptr, ptrW, cbRetBuffer - length - 1,
+		  &len);
+	      length += len;
+	    }
+
+	  *ptrW = L'\0';
+	  length++;
+	}
+      else
+	{
+	  dm_StrCopyOut2_U8toW (_buffer_u8, lpszRetBuffer, cbRetBuffer,
+	      &length);
+	}
+    }
+  else
+    {
+      dm_StrCopyOut2_U8toW (_buffer_u8, lpszRetBuffer, cbRetBuffer, &length);
+    }
+
+done:
+  MEM_FREE (_section_u8);
+  MEM_FREE (_entry_u8);
+  MEM_FREE (_default_u8);
+  MEM_FREE (_buffer_u8);
+  MEM_FREE (_filename_u8);
+
+  return length;
+}
+
+BOOL INSTAPI
+SQLGetKeywordValue (LPCSTR lpszSection,
+    LPCSTR lpszEntry, LPSTR lpszBuffer, int cbBuffer, int *pcbBufOut)
+{
+  int ret =
+      SQLGetPrivateProfileString (lpszSection, lpszEntry, "", lpszBuffer,
+      cbBuffer, "odbc.ini");
+
+  if (pcbBufOut)
+    *pcbBufOut = ret;
+
+  return (ret != 0);
+}
+
+BOOL INSTAPI
+SQLGetKeywordValueW (LPCWSTR lpszSection,
+    LPCWSTR lpszEntry, LPWSTR lpszBuffer, int cbBuffer, int *pcbBufOut)
+{
+  int ret =
+      SQLGetPrivateProfileStringW (lpszSection, lpszEntry, L"", lpszBuffer,
+      cbBuffer, L"odbc.ini");
+
+  if (pcbBufOut)
+    *pcbBufOut = ret;
+
+  return (ret != 0);
+}
