@@ -1,7 +1,7 @@
 /*
  *  SQLInstallDriver.c
  *
- *  $Id: SQLInstallDriver.c,v 1.5 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLInstallDriver.c,v 1.9 2005/09/12 08:28:43 source Exp $
  *
  *  Install a driver
  *
@@ -71,7 +71,8 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "misc.h"
 #include "inifile.h"
@@ -91,7 +92,7 @@
 
 
 BOOL
-InstallDriverPath (LPSTR lpszPath, WORD cbPathMax, WORD *pcbPathOut,
+InstallDriverPath (LPSTR lpszPath, WORD cbPathMax, WORD * pcbPathOut,
     LPSTR envname)
 {
 #ifdef _MAC
@@ -228,7 +229,7 @@ quit:
 
 BOOL INSTAPI
 SQLInstallDriver (LPCSTR lpszInfFile, LPCSTR lpszDriver, LPSTR lpszPath,
-    WORD cbPathMax, WORD *pcbPathOut)
+    WORD cbPathMax, WORD * pcbPathOut)
 {
   PCONFIG pCfg = NULL, pOdbcCfg = NULL;
   BOOL retcode = FALSE;
@@ -262,7 +263,7 @@ SQLInstallDriver (LPCSTR lpszInfFile, LPCSTR lpszDriver, LPSTR lpszPath,
     case ODBC_SYSTEM_DSN:
       wSystemDSN = SYSTEMDSN_ONLY;
       break;
-    };
+    }
 
   if (_iodbcdm_cfg_search_init (&pCfg, "odbcinst.ini", TRUE))
     {
@@ -311,3 +312,52 @@ quit:
 
   return retcode;
 }
+
+BOOL INSTAPI
+SQLInstallDriverW (LPCWSTR lpszInfFile, LPCWSTR lpszDriver, LPWSTR lpszPath,
+    WORD cbPathMax, WORD FAR * pcbPathOut)
+{
+  char *_inf_u8 = NULL;
+  char *_driver_u8 = NULL;
+  char *_path_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  _inf_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszInfFile, SQL_NTS);
+  if (_inf_u8 == NULL && lpszInfFile)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _driver_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszDriver, SQL_NTS);
+  if (_driver_u8 == NULL && lpszDriver)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  if (cbPathMax > 0)
+    {
+      if ((_path_u8 = malloc (cbPathMax * UTF8_MAX_CHAR_LEN + 1)) == NULL)
+	{
+	  PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+	  goto done;
+	}
+    }
+
+  retcode =
+      SQLInstallDriver (_inf_u8, _driver_u8, _path_u8,
+      cbPathMax * UTF8_MAX_CHAR_LEN, pcbPathOut);
+
+  if (retcode == TRUE)
+    {
+      dm_StrCopyOut2_U8toW (_path_u8, lpszPath, cbPathMax, pcbPathOut);
+    }
+
+done:
+  MEM_FREE (_inf_u8);
+  MEM_FREE (_driver_u8);
+  MEM_FREE (_path_u8);
+
+  return retcode;
+}
