@@ -1,7 +1,7 @@
 /*
  *  SQLInstallDriverManager.c
  *
- *  $Id: SQLInstallDriverManager.c,v 1.4 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLInstallDriverManager.c,v 1.7 2005/05/12 12:27:15 source Exp $
  *
  *  These functions intentionally left blank
  *
@@ -71,16 +71,16 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "iodbc_error.h"
 
-extern BOOL InstallDriverPath ( LPSTR lpszPath, WORD cbPathMax,
-    WORD * pcbPathOut,LPSTR envname);
-
+extern BOOL InstallDriverPath (LPSTR lpszPath, WORD cbPathMax,
+    WORD * pcbPathOut, LPSTR envname);
 
 BOOL INSTAPI
-SQLInstallDriverManager (LPSTR lpszPath, WORD cbPathMax, WORD *pcbPathOut)
+SQLInstallDriverManager (LPSTR lpszPath, WORD cbPathMax, WORD * pcbPathOut)
 {
   BOOL retcode = FALSE;
 
@@ -98,3 +98,34 @@ SQLInstallDriverManager (LPSTR lpszPath, WORD cbPathMax, WORD *pcbPathOut)
 quit:
   return retcode;
 }
+
+BOOL INSTAPI
+SQLInstallDriverManagerW (LPWSTR lpszPath, WORD cbPathMax,
+    WORD FAR * pcbPathOut)
+{
+  char *_path_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  if (cbPathMax > 0)
+    {
+      if ((_path_u8 = malloc (cbPathMax * UTF8_MAX_CHAR_LEN + 1)) == NULL)
+	{
+	  PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+	  goto done;
+	}
+    }
+
+  retcode =
+      SQLInstallDriverManager (_path_u8, cbPathMax * UTF8_MAX_CHAR_LEN,
+      pcbPathOut);
+
+  if (retcode == TRUE)
+    {
+      dm_StrCopyOut2_U8toW (_path_u8, lpszPath, cbPathMax, pcbPathOut);
+    }
+
+done:
+  MEM_FREE (_path_u8);
+
+  return retcode;
+}
