@@ -1,7 +1,7 @@
 /*
  *  SQLInstallTranslator.c
  *
- *  $Id: SQLInstallTranslator.c,v 1.5 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLInstallTranslator.c,v 1.9 2005/09/12 08:28:43 source Exp $
  *
  *  These functions intentionally left blank
  *
@@ -71,7 +71,8 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "misc.h"
 #include "inifile.h"
@@ -83,16 +84,15 @@
 # define UNIX_PWD
 #endif
 
-
 extern BOOL InstallDriverPath (LPSTR lpszPath, WORD cbPathMax,
-    WORD *pcbPathOut,LPSTR envname);
-extern BOOL InstallDriverPathLength (WORD * pcbPathOut,LPSTR envname);
+    WORD * pcbPathOut, LPSTR envname);
+extern BOOL InstallDriverPathLength (WORD * pcbPathOut, LPSTR envname);
 
 
 BOOL INSTAPI
 SQLInstallTranslator (LPCSTR lpszInfFile, LPCSTR lpszTranslator,
     LPCSTR lpszPathIn, LPSTR lpszPathOut, WORD cbPathOutMax,
-    WORD *pcbPathOut, WORD fRequest, LPDWORD lpdwUsageCount)
+    WORD * pcbPathOut, WORD fRequest, LPDWORD lpdwUsageCount)
 {
   PCONFIG pCfg = NULL, pOdbcCfg = NULL;
   BOOL retcode = FALSE;
@@ -125,7 +125,7 @@ SQLInstallTranslator (LPCSTR lpszInfFile, LPCSTR lpszTranslator,
     default:
       PUSH_ERROR (ODBC_ERROR_INVALID_REQUEST_TYPE);
       goto quit;
-    };
+    }
 
   /* Check input parameters */
   if (!lpszTranslator || !STRLEN (lpszTranslator))
@@ -156,7 +156,7 @@ SQLInstallTranslator (LPCSTR lpszInfFile, LPCSTR lpszTranslator,
     case ODBC_SYSTEM_DSN:
       wSystemDSN = SYSTEMDSN_ONLY;
       break;
-    };
+    }
 
   if (_iodbcdm_cfg_search_init (&pCfg, "odbcinst.ini", TRUE))
     {
@@ -206,3 +206,65 @@ quit:
 
   return retcode;
 }
+
+BOOL INSTAPI
+SQLInstallTranslatorW (LPCWSTR lpszInfFile, LPCWSTR lpszTranslator,
+    LPCWSTR lpszPathIn, LPWSTR lpszPathOut, WORD cbPathOutMax,
+    WORD FAR * pcbPathOut, WORD fRequest, LPDWORD lpdwUsageCount)
+{
+  char *_inf_u8 = NULL;
+  char *_translator_u8 = NULL;
+  char *_pathin_u8 = NULL;
+  char *_pathout_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  _inf_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszInfFile, SQL_NTS);
+  if (_inf_u8 == NULL && lpszInfFile)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _translator_u8 =
+      (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszTranslator, SQL_NTS);
+  if (_translator_u8 == NULL && lpszTranslator)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _pathin_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszPathIn, SQL_NTS);
+  if (_pathin_u8 == NULL && lpszPathIn)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  if (cbPathOutMax > 0)
+    {
+      if ((_pathout_u8 =
+	      malloc (cbPathOutMax * UTF8_MAX_CHAR_LEN + 1)) == NULL)
+	{
+	  PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+	  goto done;
+	}
+    }
+
+  retcode = SQLInstallTranslator (_inf_u8, _translator_u8, _pathin_u8,
+      _pathout_u8, cbPathOutMax * UTF8_MAX_CHAR_LEN, pcbPathOut, fRequest,
+      lpdwUsageCount);
+
+  if (retcode == TRUE)
+    {
+      dm_StrCopyOut2_U8toW (_pathout_u8, lpszPathOut, cbPathOutMax,
+	  pcbPathOut);
+    }
+
+done:
+  MEM_FREE (_inf_u8);
+  MEM_FREE (_translator_u8);
+  MEM_FREE (_pathin_u8);
+  MEM_FREE (_pathout_u8);
+
+  return retcode;
+}
