@@ -1,7 +1,7 @@
 /*
  *  SQLPostInstallerError.c
  *
- *  $Id: SQLPostInstallerError.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLPostInstallerError.c,v 1.6 2005/05/12 12:27:15 source Exp $
  *
  *  These functions intentionally left blank
  *
@@ -71,11 +71,11 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "iodbc_error.h"
 
-
 RETCODE INSTAPI
 SQLPostInstallerError (DWORD fErrorCode, LPSTR szErrorMsg)
 {
@@ -97,3 +97,24 @@ SQLPostInstallerError (DWORD fErrorCode, LPSTR szErrorMsg)
 quit:
   return retcode;
 }
+
+RETCODE INSTAPI
+SQLPostInstallerErrorW (DWORD fErrorCode, LPWSTR szErrorMsg)
+{
+  char *_errormsg_u8 = NULL;
+  RETCODE retcode = SQL_ERROR;
+
+  _errormsg_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) szErrorMsg, SQL_NTS);
+  if (_errormsg_u8 == NULL && szErrorMsg)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  retcode = SQLPostInstallerError (fErrorCode, _errormsg_u8);
+
+done:
+  MEM_FREE (_errormsg_u8);
+
+  return retcode;
+}
