@@ -1,7 +1,7 @@
 /*
- *  SQLRemoveDSNFromIni.c
+ *  SQLRemoveDefaultDataSource.c
  *
- *  $Id: SQLRemoveDefaultDataSource.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLRemoveDefaultDataSource.c,v 1.5 2005/05/12 12:27:15 source Exp $
  *
  *  Remove a DSN
  *
@@ -71,7 +71,7 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 BOOL INSTAPI
 SQLRemoveDefaultDataSource (void)
