@@ -1,7 +1,7 @@
 /*
  *  SQLRemoveDriver.c
  *
- *  $Id: SQLRemoveDriver.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLRemoveDriver.c,v 1.7 2005/09/12 08:28:43 source Exp $
  *
  *  Remove a driver
  *
@@ -71,7 +71,8 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "inifile.h"
 #include "misc.h"
@@ -106,7 +107,7 @@ SQLRemoveDriver (LPCSTR lpszDriver, BOOL fRemoveDSN, LPDWORD lpdwUsageCount)
     case ODBC_SYSTEM_DSN:
       wSystemDSN = SYSTEMDSN_ONLY;
       break;
-    };
+    }
 
   if (_iodbcdm_cfg_search_init (&pCfg, "odbc.ini", FALSE))
     {
@@ -223,3 +224,24 @@ quit:
 
   return retcode;
 }
+
+BOOL INSTAPI
+SQLRemoveDriverW (LPCWSTR lpszDriver, BOOL fRemoveDSN, LPDWORD lpdwUsageCount)
+{
+  char *_driver_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  _driver_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszDriver, SQL_NTS);
+  if (_driver_u8 == NULL && lpszDriver)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  retcode = SQLRemoveDriver (_driver_u8, fRemoveDSN, lpdwUsageCount);
+
+done:
+  MEM_FREE (_driver_u8);
+
+  return retcode;
+}
