@@ -1,7 +1,7 @@
 /*
  *  SQLRemoveDriverManager.c
  *
- *  $Id: SQLRemoveDriverManager.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLRemoveDriverManager.c,v 1.4 2005/05/12 12:27:15 source Exp $
  *
  *  These functions intentionally left blank
  *
@@ -70,7 +70,7 @@
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include "inifile.h"
 #include "misc.h"
