@@ -1,7 +1,7 @@
 /*
  *  SQLValidDSN.c
  *
- *  $Id: SQLValidDSN.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: SQLValidDSN.c,v 1.6 2005/05/12 12:27:15 source Exp $
  *
  *  Validate a DSN name
  *
@@ -71,12 +71,13 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "iodbc_error.h"
 
 #define INVALID_CHARS	"[]{}(),;?*=!@\\"
-
+#define INVALID_CHARSW	L"[]{}(),;?*=!@\\"
 
 BOOL
 ValidDSN (LPCSTR lpszDSN)
@@ -88,7 +89,24 @@ ValidDSN (LPCSTR lpszDSN)
       if (strchr (INVALID_CHARS, *currp))
 	return FALSE;
       else
-	currp += 1;
+	currp++;
+    }
+
+  return TRUE;
+}
+
+
+BOOL
+ValidDSNW (LPCWSTR lpszDSN)
+{
+  wchar_t *currp = (wchar_t *) lpszDSN;
+
+  while (*currp)
+    {
+      if (wcschr (INVALID_CHARSW, *currp))
+	return FALSE;
+      else
+	currp++;
     }
 
   return TRUE;
@@ -102,7 +120,7 @@ SQLValidDSN (LPCSTR lpszDSN)
 
   /* Check dsn */
   CLEAR_ERROR ();
-  if (!lpszDSN || !strlen (lpszDSN) || strlen (lpszDSN) >= SQL_MAX_DSN_LENGTH)
+  if (!lpszDSN || !STRLEN (lpszDSN) || STRLEN (lpszDSN) >= SQL_MAX_DSN_LENGTH)
     {
       PUSH_ERROR (ODBC_ERROR_GENERAL_ERR);
       goto quit;
@@ -113,3 +131,22 @@ SQLValidDSN (LPCSTR lpszDSN)
 quit:
   return retcode;
 }
+
+BOOL INSTAPI
+SQLValidDSNW (LPCWSTR lpszDSN)
+{
+  BOOL retcode = FALSE;
+
+  /* Check dsn */
+  CLEAR_ERROR ();
+  if (!lpszDSN || !WCSLEN (lpszDSN) || WCSLEN (lpszDSN) >= SQL_MAX_DSN_LENGTH)
+    {
+      PUSH_ERROR (ODBC_ERROR_GENERAL_ERR);
+      goto quit;
+    }
+
+  retcode = ValidDSNW (lpszDSN);
+
+quit:
+  return retcode;
+}
