@@ -1,19 +1,19 @@
 /*
  *  SQLWriteFileDSN.c
  *
- *  $Id: SQLWriteFileDSN.c,v 1.2 2002/03/14 20:53:44 source Exp $
+ *  $Id: SQLWriteFileDSN.c,v 1.5 2005/05/12 12:27:15 source Exp $
  *
  *  These functions intentionally left blank
  *
  *  The iODBC driver manager.
- *  
+ *
  *  Copyright (C) 1999-2002 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
  *  licenses:
  *
- *      - GNU Library General Public License (see LICENSE.LGPL) 
+ *      - GNU Library General Public License (see LICENSE.LGPL)
  *      - The BSD License (see LICENSE.BSD).
  *
  *  While not mandated by the BSD license, any patches you make to the
@@ -71,7 +71,8 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "inifile.h"
 #include "iodbc_error.h"
@@ -104,3 +105,53 @@ SQLWriteFileDSN (LPCSTR lpszFileName, LPCSTR lpszAppName, LPCSTR lpszKeyName,
 quit:
   return retcode;
 }
+
+BOOL INSTAPI
+SQLWriteFileDSNW (LPCWSTR lpszFileName, LPCWSTR lpszAppName,
+    LPCWSTR lpszKeyName, LPWSTR lpszString)
+{
+  char *_filename_u8 = NULL;
+  char *_appname_u8 = NULL;
+  char *_keyname_u8 = NULL;
+  char *_string_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  _filename_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszFileName, SQL_NTS);
+  if (_filename_u8 == NULL && lpszFileName)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _appname_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszAppName, SQL_NTS);
+  if (_appname_u8 == NULL && lpszAppName)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _keyname_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszKeyName, SQL_NTS);
+  if (_keyname_u8 == NULL && lpszKeyName)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _string_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszString, SQL_NTS);
+  if (_string_u8 == NULL && lpszString)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  retcode =
+      SQLWriteFileDSN (_filename_u8, _appname_u8, _keyname_u8, _string_u8);
+
+done:
+  MEM_FREE (_filename_u8);
+  MEM_FREE (_appname_u8);
+  MEM_FREE (_keyname_u8);
+  MEM_FREE (_string_u8);
+
+  return retcode;
+}
