@@ -1,17 +1,17 @@
 /*
  *  SQLWritePrivateProfileString.c
  *
- *  $Id: SQLWritePrivateProfileString.c,v 1.2 2002/03/14 20:53:44 source Exp $
+ *  $Id: SQLWritePrivateProfileString.c,v 1.6 2005/09/12 08:28:43 source Exp $
  *
  *  The iODBC driver manager.
- *  
+ *
  *  Copyright (C) 1999-2002 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
  *  licenses:
  *
- *      - GNU Library General Public License (see LICENSE.LGPL) 
+ *      - GNU Library General Public License (see LICENSE.LGPL)
  *      - The BSD License (see LICENSE.BSD).
  *
  *  While not mandated by the BSD license, any patches you make to the
@@ -69,7 +69,8 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
+#include <unicode.h>
 
 #include "inifile.h"
 #include "iodbc_error.h"
@@ -206,7 +207,7 @@ SQLWritePrivateProfileString (LPCSTR lpszSection, LPCSTR lpszEntry,
 		pathbuf);
 	}
       goto quit;
-    };
+    }
 
   PUSH_ERROR (ODBC_ERROR_GENERAL_ERR);
   goto quit;
@@ -216,3 +217,70 @@ quit:
   configMode = ODBC_BOTH_DSN;
   return retcode;
 }
+
+BOOL INSTAPI
+SQLWritePrivateProfileStringW (LPCWSTR lpszSection, LPCWSTR lpszEntry,
+    LPCWSTR lpszString, LPCWSTR lpszFilename)
+{
+  char *_section_u8 = NULL;
+  char *_entry_u8 = NULL;
+  char *_string_u8 = NULL;
+  char *_filename_u8 = NULL;
+  BOOL retcode = FALSE;
+
+  _section_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszSection, SQL_NTS);
+  if (_section_u8 == NULL && lpszSection)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _entry_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszEntry, SQL_NTS);
+  if (_entry_u8 == NULL && lpszEntry)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _string_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszString, SQL_NTS);
+  if (_string_u8 == NULL && lpszString)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  _filename_u8 = (char *) dm_SQL_WtoU8 ((SQLWCHAR *) lpszFilename, SQL_NTS);
+  if (_filename_u8 == NULL && lpszFilename)
+    {
+      PUSH_ERROR (ODBC_ERROR_OUT_OF_MEM);
+      goto done;
+    }
+
+  retcode =
+      SQLWritePrivateProfileString (_section_u8, _entry_u8, _string_u8,
+      _filename_u8);
+
+done:
+  MEM_FREE (_section_u8);
+  MEM_FREE (_entry_u8);
+  MEM_FREE (_string_u8);
+  MEM_FREE (_filename_u8);
+
+  return retcode;
+}
+
+BOOL INSTAPI
+SQLSetKeywordValue (LPCSTR lpszSection,
+    LPCSTR lpszEntry, LPSTR lpszString, int cbString)
+{
+  return SQLWritePrivateProfileString (lpszSection,
+      lpszEntry, lpszString, "odbc.ini");
+}
+
+BOOL INSTAPI
+SQLSetKeywordValueW (LPCWSTR lpszSection,
+    LPCWSTR lpszEntry, LPWSTR lpszString, int cbString)
+{
+  return SQLWritePrivateProfileStringW (lpszSection,
+      lpszEntry, lpszString, L"odbc.ini");
+}
