@@ -1,7 +1,7 @@
 /*
  *  inifile.c
  *
- *  $Id: inifile.c,v 1.4 2004/07/13 13:11:31 source Exp $
+ *  $Id: inifile.c,v 1.5 2005/05/12 12:27:15 source Exp $
  *
  *  Configuration File Management
  *
@@ -71,7 +71,7 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include <stdarg.h>
 #include <stdio.h>
