@@ -1,7 +1,7 @@
 /*
  *  iodbc_error.h
  *
- *  $Id: iodbc_error.c,v 1.3 2004/06/22 11:19:20 source Exp $
+ *  $Id: iodbc_error.c,v 1.4 2005/05/12 12:27:15 source Exp $
  *
  *  Misc support functions
  *
@@ -71,7 +71,7 @@
  */
 
 #include <iodbc.h>
-#include <iodbcinst.h>
+#include <odbcinst.h>
 
 #include "iodbc_error.h"
 
