@@ -1,7 +1,7 @@
 /*
  *  iodbctest.c
  *
- *  $Id: iodbctest.c,v 1.20 2004/12/15 15:31:05 source Exp $
+ *  $Id: iodbctest.c,v 1.24 2005/10/08 11:43:32 source Exp $
  *
  *  Sample ODBC program
  *
@@ -172,6 +172,9 @@ strcpy_A2W (SQLWCHAR * destStr, char *sourStr)
  *
  *	DSN=pubs_sqlserver;PWD=demo
  */
+
+SQLTCHAR outdsn[4096];			/* Store completed DSN for later use */
+
 int
 ODBC_Connect (char *connStr)
 {
@@ -180,7 +183,6 @@ ODBC_Connect (char *connStr)
   SQLTCHAR dsn[33];
   SQLTCHAR desc[255];
   SQLTCHAR driverInfo[255];
-  SQLTCHAR outdsn[1024];
   SQLSMALLINT len1, len2;
   int status;
 #ifdef UNICODE
@@ -252,7 +254,7 @@ ODBC_Connect (char *connStr)
 	/*
 	 * Check if the user wants to quit
 	 */
-	if (!strcmp (dataSource, "quit") || !strcmp (dataSource, "exit"))
+	if (!strcmp ((char *)dataSource, "quit") || !strcmp ((char *)dataSource, "exit"))
 	  return -1;
 
 	/*
@@ -297,12 +299,12 @@ ODBC_Connect (char *connStr)
 #ifdef UNICODE
   strcpy_A2W (wdataSource, (char *) dataSource);
   status = SQLDriverConnectW (hdbc, 0, (SQLWCHAR *) wdataSource, SQL_NTS,
-      (SQLWCHAR *) outdsn, NUMTCHAR (outdsn), &buflen, SQL_DRIVER_PROMPT);
+      (SQLWCHAR *) outdsn, NUMTCHAR (outdsn), &buflen, SQL_DRIVER_COMPLETE);
   if (status != SQL_SUCCESS)
     ODBC_Errors ("SQLDriverConnectW");
 #else
   status = SQLDriverConnect (hdbc, 0, (SQLCHAR *) dataSource, SQL_NTS,
-      (SQLCHAR *) outdsn, NUMTCHAR (outdsn), &buflen, SQL_DRIVER_PROMPT);
+      (SQLCHAR *) outdsn, NUMTCHAR (outdsn), &buflen, SQL_DRIVER_COMPLETE);
   if (status != SQL_SUCCESS)
     ODBC_Errors ("SQLDriverConnect");
 #endif
@@ -413,24 +415,88 @@ ODBC_Disconnect (void)
 }
 
 
+/*
+ *  Perform a disconnect/reconnect using the DSN stored from the original
+ *  SQLDriverConnect
+ */
+int 
+ODBC_Reconnect (void)
+{
+  SQLRETURN status;
+  SQLTCHAR buf[4096];
+  SQLSMALLINT len;
+
+  /*
+   *  Free old statement handle
+   */
+#if (ODBCVER < 0x0300)
+  SQLFreeStmt (hstmt, SQL_DROP);
+#else
+  SQLFreeHandle (SQL_HANDLE_STMT, hstmt);
+#endif
+
+  /*
+   *  Disconnect
+   */
+  SQLDisconnect (hdbc);
+
+  /*
+   *  Reconnect
+   */
+  status = SQLDriverConnect (hdbc, 0, outdsn, SQL_NTS,
+      buf, sizeof (buf), &len, SQL_DRIVER_NOPROMPT);
+
+  /*
+   *  Allocate new statement handle
+   */
+  if (SQL_SUCCEEDED (status))
+    {
+#if (ODBCVER < 0x0300)
+      status = SQLAllocStmt (hdbc, &hstmt);
+#else
+      status = SQLAllocHandle (SQL_HANDLE_STMT, hdbc, &hstmt);
+#endif
+    }
+
+  /*
+   *  Show why we where unsuccessful and return an error
+   */
+  if (!SQL_SUCCEEDED (status))
+    {
+      ODBC_Errors ("DriverConnect (reconnect)");
+      return -1;
+    }
+
+  /*
+   *  Success
+   */
+  return 0;
+}
+
+
 /*
  *  Show all the error information that is available
  */
 int
 ODBC_Errors (char *where)
 {
-  SQLTCHAR buf[250];
+  SQLTCHAR buf[512];
   SQLTCHAR sqlstate[15];
   SQLINTEGER native_error = 0;
   int force_exit = 0;
+  SQLRETURN sts;
 
 #if (ODBCVER < 0x0300)
   /*
    *  Get statement errors
    */
-  while (hstmt && SQLError (henv, hdbc, hstmt, sqlstate, &native_error,
-	  buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (hstmt)
     {
+      sts = SQLError (henv, hdbc, hstmt, sqlstate, &native_error,
+	  buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%s = %S (%ld) SQLSTATE=%S\n",
 	  where, buf, (long) native_error, sqlstate);
@@ -450,9 +516,13 @@ ODBC_Errors (char *where)
   /*
    *  Get connection errors
    */
-  while (hdbc && SQLError (henv, hdbc, SQL_NULL_HSTMT, sqlstate, &native_error,
-	  buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (hdbc)
     {
+      sts = SQLError (henv, hdbc, SQL_NULL_HSTMT, sqlstate, &native_error,
+	  buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%s = %S (%ld) SQLSTATE=%S\n",
 	  where, buf, (long) native_error, sqlstate);
@@ -472,9 +542,13 @@ ODBC_Errors (char *where)
   /*
    *  Get environment errors
    */
-  while (henv && SQLError (henv, SQL_NULL_HDBC, SQL_NULL_HSTMT, sqlstate,
-	  &native_error, buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (henv)
     {
+      sts SQLError (henv, SQL_NULL_HDBC, SQL_NULL_HSTMT, sqlstate,
+	  &native_error, buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%s = %S (%ld) SQLSTATE=%S\n",
 	  where, buf, (long) native_error, sqlstate);
@@ -497,9 +571,13 @@ ODBC_Errors (char *where)
    *  Get statement errors
    */
   i = 0;
-  while (hstmt && i < 5 && SQLGetDiagRec (SQL_HANDLE_STMT, hstmt, ++i,
-	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (hstmt && i < 5)
     {
+      sts = SQLGetDiagRec (SQL_HANDLE_STMT, hstmt, ++i,
+	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%d: %s = %S (%ld) SQLSTATE=%S\n",
 	  i, where, buf, (long) native_error, sqlstate);
@@ -520,9 +598,13 @@ ODBC_Errors (char *where)
    *  Get connection errors
    */
   i = 0;
-  while (hdbc && i < 5 && SQLGetDiagRec (SQL_HANDLE_DBC, hdbc, ++i,
-	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (hdbc && i < 5)
     {
+      sts = SQLGetDiagRec (SQL_HANDLE_DBC, hdbc, ++i,
+	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%d: %s = %S (%ld) SQLSTATE=%S\n",
 	  i, where, buf, (long) native_error, sqlstate);
@@ -543,9 +625,13 @@ ODBC_Errors (char *where)
    *  Get environment errors
    */
   i = 0;
-  while (henv && i < 5 && SQLGetDiagRec (SQL_HANDLE_ENV, henv, ++i,
-	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL) == SQL_SUCCESS)
+  while (henv && i < 5)
     {
+      sts = SQLGetDiagRec (SQL_HANDLE_ENV, henv, ++i,
+	  sqlstate, &native_error, buf, NUMTCHAR (buf), NULL);
+      if (!SQL_SUCCEEDED (sts))
+	break;
+
 #ifdef UNICODE
       fprintf (stderr, "%d: %s = %S (%ld) SQLSTATE=%S\n",
 	  i, where, buf, (long) native_error, sqlstate);
@@ -676,6 +762,13 @@ ODBC_Test ()
 	      continue;
 	    }
 	}
+      else if (!TXTCMP (request, TEXT ("reconnect")))
+	{
+  	  if (ODBC_Reconnect())
+	    return -1;
+
+	  continue;
+	}
 #if defined (unix)
       else if (!TXTCMP (request, TEXT ("environment")))
 	{
